import { GraphQLNamedType, GraphQLAbstractType, GraphQLObjectType } from 'graphql/type/definition';
import { SchemaDefinitionNode, SchemaExtensionNode } from 'graphql/language/ast';
import { GraphQLDirective } from 'graphql/type/directives';
declare type Maybe<T> = null | undefined | T;
export declare class GraphQLSchema {
    astNode: Maybe<SchemaDefinitionNode>;
    extensionASTNodes: Maybe<ReadonlyArray<SchemaExtensionNode>>;
    private _queryType;
    private _mutationType;
    private _subscriptionType;
    private _directives;
    private _typeMap;
    private _implementations;
    private _possibleTypeMap;
    private __validationErrors;
    private __allowedLegacyNames;
    constructor(config: GraphQLSchemaConfig);
    getQueryType(): Maybe<GraphQLObjectType>;
    getMutationType(): Maybe<GraphQLObjectType>;
    getSubscriptionType(): Maybe<GraphQLObjectType>;
    getTypeMap(): TypeMap;
    getType(name: string): Maybe<GraphQLNamedType>;
    getPossibleTypes(abstractType: GraphQLAbstractType): ReadonlyArray<GraphQLObjectType>;
    isPossibleType(abstractType: GraphQLAbstractType, possibleType: GraphQLObjectType): boolean;
    getDirectives(): ReadonlyArray<GraphQLDirective>;
    getDirective(name: string): Maybe<GraphQLDirective>;
}
declare type TypeMap = Record<string, GraphQLNamedType>;
export interface GraphQLSchemaValidationOptions {
    assumeValid?: boolean;
    allowedLegacyNames?: Maybe<ReadonlyArray<string>>;
}
export interface GraphQLSchemaConfig extends GraphQLSchemaValidationOptions {
    query?: Maybe<GraphQLObjectType>;
    mutation?: Maybe<GraphQLObjectType>;
    subscription?: Maybe<GraphQLObjectType>;
    types?: Maybe<Array<GraphQLNamedType>>;
    directives?: Maybe<Array<GraphQLDirective>>;
    astNode?: Maybe<SchemaDefinitionNode>;
    extensionASTNodes?: Maybe<ReadonlyArray<SchemaExtensionNode>>;
}
export {};
//# sourceMappingURL=GraphQLSchema.d.ts.map