"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const validate_1 = require("graphql/type/validate");
const parser_1 = require("graphql/language/parser");
const validate_2 = require("graphql/validation/validate");
const execute_1 = require("../execute");
function graphql(argsOrSchema, source, rootValue, contextValue, variableValues, operationName, fieldResolver) {
    return new Promise(resolve => resolve(arguments.length === 1
        ? graphqlImpl(argsOrSchema.schema, argsOrSchema.source, argsOrSchema.rootValue, argsOrSchema.contextValue, argsOrSchema.variableValues, argsOrSchema.operationName, argsOrSchema.fieldResolver)
        : graphqlImpl(argsOrSchema, source, rootValue, contextValue, variableValues, operationName, fieldResolver)));
}
exports.graphql = graphql;
function graphqlSync(argsOrSchema, source, rootValue, contextValue, variableValues, operationName, fieldResolver) {
    const result = arguments.length === 1
        ? graphqlImpl(argsOrSchema.schema, argsOrSchema.source, argsOrSchema.rootValue, argsOrSchema.contextValue, argsOrSchema.variableValues, argsOrSchema.operationName, argsOrSchema.fieldResolver)
        : graphqlImpl(argsOrSchema, source, rootValue, contextValue, variableValues, operationName, fieldResolver);
    if (result.then) {
        throw new Error('GraphQL execution failed to complete synchronously.');
    }
    return result;
}
exports.graphqlSync = graphqlSync;
function graphqlImpl(schema, source, rootValue, contextValue, variableValues, operationName, fieldResolver) {
    const schemaValidationErrors = validate_1.validateSchema(schema);
    if (schemaValidationErrors.length > 0) {
        return { errors: schemaValidationErrors };
    }
    let document;
    try {
        document = parser_1.parse(source);
    }
    catch (syntaxError) {
        return { errors: [syntaxError] };
    }
    const validationErrors = validate_2.validate(schema, document);
    if (validationErrors.length > 0) {
        return { errors: validationErrors };
    }
    return execute_1.execute(schema, document, rootValue, contextValue, variableValues, operationName, fieldResolver);
}
//# sourceMappingURL=graphql.js.map